/**
 * FastReverseDNS batch reverse DNS lookup
 * Copyright (C) 2002 Jesse Wilson <jesse@swank.ca>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ca.swank.fastreversedns;

import java.util.*;
import java.net.*;


/**
 * A range of IP addresses that can be enumerated in sequence
 */
public class IPEnumeration implements Enumeration
{
  byte[] currentAddress;
  byte[] lastAddress;

  /**
   * Creates a new IPEnumeration starting with the firstIP and ending with
   * the lastIP (inclusive).
   */
  public IPEnumeration(String firstIp, String lastIp)
  { try
    { currentAddress = InetAddress.getByName(firstIp).getAddress();
      lastAddress = InetAddress.getByName(lastIp).getAddress();
    } catch(UnknownHostException e)
    { currentAddress = null;
      lastAddress = null;
    }
  }


  /**
   * returns true if the current address is less than the last address
   */
  public boolean hasMoreElements()
  { for(int i = 0; i < currentAddress.length; i++)
    { if((currentAddress[i] + 256) % 256 > (lastAddress[i] + 256) % 256) return false;
      else if((currentAddress[i] + 256) % 256 < (lastAddress[i] + 256) % 256) return true;
    }
    return true;
  }


  /**
   * returns the current address as a String or null and increments the address counter
   */
  public Object nextElement()
  { try
    { String result = InetAddress.getByAddress(currentAddress).getHostAddress();

      // increment current address
      boolean carry = true;
      int pos = currentAddress.length - 1;
      while(carry)
      { if((currentAddress[pos] + 256) % 256 == 255)
        { currentAddress[pos] = 0;
          carry = true;
          pos = pos - 1;
        } else
        { currentAddress[pos] = (byte)(currentAddress[pos] + 256 + 1);
          carry = false;
        }
      }

      // now return the result
      return result;

    } catch(UnknownHostException e)
    { return null;
    }
  }

}